"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const color_1 = require("@heroku-cli/color");
const base_1 = tslib_1.__importDefault(require("../../lib/base"));
const command_1 = require("@heroku-cli/command");
const core_1 = require("@oclif/core");
const open_1 = tslib_1.__importDefault(require("open"));
const helpers_1 = require("../../lib/helpers");
const tsheredoc_1 = tslib_1.__importDefault(require("tsheredoc"));
class Connect extends base_1.default {
    static description = 'connects a Data Cloud Org to a Heroku app';
    static flags = {
        app: command_1.flags.app({ required: true }),
        browser: command_1.flags.string({ description: 'browser to open OAuth flow with (example: "firefox", "safari")' }),
        'login-url': command_1.flags.string({ char: 'l', description: 'login URL' }),
        remote: command_1.flags.remote(),
    };
    static args = {
        org_name: core_1.Args.string({ description: 'Data Cloud Org instance name.  Must begin with a letter. Then allowed chars are alphanumeric and underscores \'_\' (non-consecutive). Must end with a letter or a number. Must be min 3, max 30 characters.', required: true }),
    };
    static urlOpener = open_1.default;
    async run() {
        const { flags, args } = await this.parse(Connect);
        const { app, browser, 'login-url': loginUrl } = flags;
        const { org_name: orgName } = args;
        await this.configureIntegrationClient(app);
        let connection;
        ({ body: connection } = await this.integration.post(`/addons/${this.addonId}/connections/datacloud`, {
            body: {
                login_url: loginUrl,
                org_name: orgName,
            },
        }));
        const { id, redirect_uri: redirectUri } = connection;
        process.stderr.write(`Opening browser to ${redirectUri}\n`);
        let urlDisplayed = false;
        const showBrowserError = () => {
            if (!urlDisplayed)
                core_1.ux.warn('Cannot open browser.');
            urlDisplayed = true;
        };
        try {
            await core_1.ux.anykey(`Press any key to open up the browser to connect ${color_1.color.app(app)} to ${color_1.color.yellow(orgName)}, or ${color_1.color.yellow('q')} to exit`);
        }
        catch (error) {
            const { message, oclif } = error;
            core_1.ux.error(message, { exit: oclif?.exit || 1 });
        }
        const cp = await Connect.urlOpener(redirectUri, { wait: false, ...(browser ? { app: { name: browser } } : {}) });
        cp.on('error', (err) => {
            core_1.ux.warn(err);
            showBrowserError();
        });
        cp.on('close', (code) => {
            if (code !== 0)
                showBrowserError();
        });
        core_1.ux.action.start(`Connecting Data Cloud org ${color_1.color.yellow(orgName)} to ${color_1.color.app(app)}`);
        let { state, error } = connection;
        core_1.ux.action.status = (0, helpers_1.humanize)(state);
        while (this.isPendingState(state)) {
            await new Promise(resolve => {
                setTimeout(resolve, 5000);
            });
            ({ body: connection } = await this.integration.get(`/addons/${this.addonId}/connections/${id}`));
            ({ state, error } = connection);
            core_1.ux.action.status = (0, helpers_1.humanize)(state);
        }
        core_1.ux.action.stop((0, helpers_1.humanize)(state));
        if (state !== 'connected') {
            core_1.ux.error(error === undefined ?
                (0, helpers_1.humanize)(state) :
                (0, tsheredoc_1.default) `
            ${error.id}
            ${error.message}
          `, { exit: 1 });
        }
    }
    isPendingState(state) {
        return state !== 'connected' && state !== 'authentication_failed' && state !== 'connection_failed' && state !== 'disconnected';
    }
}
exports.default = Connect;
