"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const color_1 = require("@heroku-cli/color");
const command_1 = require("@heroku-cli/command");
const core_1 = require("@oclif/core");
const crypto_1 = require("crypto");
const fs_1 = tslib_1.__importDefault(require("fs"));
const zlib_1 = require("zlib");
const base_1 = tslib_1.__importDefault(require("../../lib/base"));
const helpers_1 = require("../../lib/helpers");
const tsheredoc_1 = tslib_1.__importDefault(require("tsheredoc"));
class Import extends base_1.default {
    static description = 'Imports an API specification to an authenticated Salesforce Org.';
    static flags = {
        app: command_1.flags.app({ required: true }),
        'client-name': command_1.flags.string({ required: true, char: 'c', description: 'name given to the client stub' }),
        'generate-auth-permission-set': command_1.flags.boolean({ char: 'G', description: 'generate a permission set for the client' }),
        'org-name': command_1.flags.string({ required: true, char: 'o', description: 'authorized Salesforce Org instance name' }),
        remote: command_1.flags.remote(),
    };
    static args = {
        api_spec_file: core_1.Args.file({ required: true, description: 'OpenAPI 3.x spec file (JSON or YAML format)' }),
    };
    isPendingState(state) {
        return state !== 'imported' && state !== 'import_failed';
    }
    async run() {
        const { flags, args } = await this.parse(Import);
        const { app, 'client-name': clientName, 'generate-auth-permission-set': generateAuthPermissionSet, 'org-name': orgName } = flags;
        const { api_spec_file: apiSpecFile } = args;
        const specFileContents = fs_1.default.readFileSync(apiSpecFile);
        const hexDigest = (0, crypto_1.createHash)('sha256').update(specFileContents).digest('hex');
        const compressedSpec = (0, zlib_1.gzipSync)(Buffer.from(specFileContents));
        const encodedSpec = Buffer.from(compressedSpec).toString('base64');
        await this.configureIntegrationClient(app);
        core_1.ux.action.start(`Importing ${color_1.color.app(app)} to ${color_1.color.yellow(orgName)} as ${color_1.color.yellow(clientName)}`);
        let importState;
        const { body: importRes } = await this.integration.post(`/addons/${this.addonId}/connections/salesforce/${orgName}/app_imports`, {
            body: {
                client_name: clientName,
                api_spec: encodedSpec,
                generate_authorization_permission_set: generateAuthPermissionSet,
                hex_digest: hexDigest,
            },
        });
        let { state, error } = importRes;
        while (this.isPendingState(state)) {
            await new Promise(resolve => {
                setTimeout(resolve, 5000);
            });
            ({ body: importState } = await this.integration.get(`/addons/${this.addonId}/connections/salesforce/${orgName}/app_imports/${clientName}`));
            // ({state, error} = importState)
            state = importState.state;
            error = importState.error;
            core_1.ux.action.status = (0, helpers_1.humanize)(state);
        }
        core_1.ux.action.stop((0, helpers_1.humanize)(state));
        if (state !== 'imported') {
            core_1.ux.error(error === undefined ?
                (0, helpers_1.humanize)(state) :
                (0, tsheredoc_1.default) `
            ${error.id}
            ${error.message}
          `, { exit: 1 });
        }
    }
}
exports.default = Import;
